.qd_tooltip {
  position: relative;
  display: inline-block;

  .qd_tooltiptext {
    visibility: hidden;
    display: block;
    width: 350px;
    word-wrap: break-word;
    background-color: rgba(0, 0, 0, 0.8);
    color: #fff;
    text-align: center;
    border-radius: 6px;
    padding: 5px;
    position: absolute;
    z-index: 1;
  }

  &:hover {
    .qd_tooltiptext {
      visibility: visible;
    }
  }
}

.qd_breakdown {
  width: 100%;

  td {
    padding-bottom: 2px;
  }
}

.qd_sidebar_recent_help {
  font-style: italic;
  & p {
    // this needs to be !important because this is a nav-item which has an !important
    // font size specified in it
    font-size: 1.1em !important;
  }

  .qd_sidebar_total {
    font-weight: bold;
  }
}

.qd_warning_banner {
  width: 100%;
  padding: 10px;
  text-align: center;
  margin-bottom: 20px;

  & > p:last-child {
    margin-bottom: 0;
  }
}

.qd_block {
  border-top: 1px solid #eee;
  border-bottom: 1px solid #eee;
  padding: 10px 0;
  margin-bottom: 40px;

  & > p:last-child {
    margin-bottom: 0;
  }
}
