.query-part {
  font-size: 1.1em;
  float: left;
}

.query-icon {
  float: left;
}

.query-q {
  clear: both;
}

.query-filters {
  clear: both;
}

.query-ul {
  list-style-type: none;
}

.query-field {
  font-style: italic;
}

.query-value {
  font-weight: bold;
}

.float-clearer {
  clear: both;
}

.visual_query {
  padding: 10px;

  &:after {
    content: '.';
    visibility: hidden;
    display: block;
    height: 0;
    clear: both;
  }

  & > pre {
    padding: 0;
  }
}
